//
//  ROJsonBinder.m
//  ROJsonBinder
//
//  Created by Raphael Oliveira on 12/5/12.
//  Copyright (c) 2012 Raphael Oliveira. All rights reserved.
//

#import "ROJsonBinder.h"
#import <objc/runtime.h>

#define TYPE_INT @"Ti"
#define TYPE_FLOAT @"Tf"
#define TYPE_DOUBLE @"Td"

@implementation ROJsonBinder

+ (id)bindClass:(Class)class withJson:(id)json
{
    NSString *className = [NSString stringWithUTF8String:class_getName(class)];
    id classInstance = [[NSClassFromString(className) alloc] init];
    
    uint propertiesLenght;
    objc_property_t *propertiesList = class_copyPropertyList(class, &propertiesLenght);

    for (uint i = 0; i < propertiesLenght; ++i)
    {
        objc_property_t property = propertiesList[i];
        NSString *propertyName = [NSString stringWithUTF8String:property_getName(property)];
        NSString *propertyType = [NSString stringWithUTF8String:property_getTypeString(property)];
        
        id object = [json objectForKey:propertyName];
        
        if ([propertyType isEqualToString:TYPE_INT])
        {
            NSNumber *number = [NSNumber numberWithInt:[object intValue]];
            [classInstance setValue:number forKey:propertyName];
        }
        else if ([propertyType isEqualToString:TYPE_FLOAT])
        {
            NSNumber *number = [NSNumber numberWithFloat:[object floatValue]];
            [classInstance setValue:number forKey:propertyName];
        }
        else if ([propertyType isEqualToString:TYPE_DOUBLE])
        {
            NSNumber *number = [NSNumber numberWithFloat:[object doubleValue]];
            [classInstance setValue:number forKey:propertyName];
        }
        else
        {
            [classInstance setValue:object forKey:propertyName];
        }
    }
    
    free(propertiesList);
    
    return classInstance;
}

const char * property_getTypeString(objc_property_t property )
{
	const char *attrs = property_getAttributes(property);
    
	if (attrs == NULL)
    {
		return (NULL);
    }
    
	static char buffer[256];
    
	const char *e = strchr(attrs, ',');
    
	if (e == NULL)
    {
		return (NULL);
    }
    
	int len = (int)(e - attrs);
	memcpy(buffer, attrs, len);
	buffer[len] = '\0';
    
	return buffer;
}

@end
