//
//  ROJsonBinderTests.m
//  ROJsonBinderTests
//
//  Created by Raphael Oliveira on 12/5/12.
//  Copyright (c) 2012 Raphael Oliveira. All rights reserved.
//

#import "ROJsonBinderTests.h"
#import "ROJsonBinder.h"
#import "Person.h"

#define FLOAT_MIN_ACCURACY 0.0001

@implementation ROJsonBinderTests

#pragma mark Protected

- (void)setUp
{
    [super setUp];
    
    self.bundle = [NSBundle bundleForClass:[self class]];
}

- (void)tearDown
{
    [super tearDown];
}

#pragma mark Public

- (id)dataFromJsonDataSetWithName:(SEL)selectorName
{
    NSString *dataSetPath = [self.bundle pathForResource:NSStringFromSelector(selectorName) ofType:@"json"];
    NSData *jsonData = [NSData dataWithContentsOfFile:dataSetPath];
    
    NSError *error;
    id json = [NSJSONSerialization JSONObjectWithData:jsonData options:NSJSONReadingMutableContainers error:&error];
    
    return json;
}

- (void)testBindClass_ReturnOnePersonInstanceWithOneAttribute_Name
{
    self.jsonDataSet = [self dataFromJsonDataSetWithName:_cmd];
    
    Person *person = [ROJsonBinder bindClass:[Person class] withJson:self.jsonDataSet];
    
    STAssertEqualObjects(person.name, @"Raphael", @"The parser should have created a Person instance with the attribute 'name' equal to 'Raphael'.");
    STAssertEquals(person.age, 0, @"The parser should have created a Person instance with the attribute 'age' equal to '0' since it does not exists in the json dataset and 0 is the default value for integers.");
    STAssertEquals(person.weight, (float)0.0, @"The parser should have created a Person instance with the attribute 'weight' equal to '0.0' since it does not exists in the json dataset and 0.0 is the default value for floats.");
    STAssertEquals(person.height, 0.0, @"The parser should have created a Person instance with the attribute 'height' equal to '0.0' since it does not exists in the json dataset and 0.0 is the default value for doubles.");
    STAssertNil(person.identityNumber, @"The parser should have created a Person instance with the attribute 'identityNumber' equal to 'nil' since it does not exists in the json dataset and nil is the default value for NSObjects.");
}

- (void)testBindClass_ReturnOnePersonInstanceWithTwoAttributes_Name_Age
{
    self.jsonDataSet = [self dataFromJsonDataSetWithName:_cmd];
    
    Person *person = [ROJsonBinder bindClass:[Person class] withJson:self.jsonDataSet];
    
    STAssertEqualObjects(person.name, @"Raphael", @"The parser should have created a Person instance with the attribute 'name' equal to 'Raphael'.");
    STAssertEquals(person.age, 23, @"The parser should have created a Person instance with the attribute 'age' equal to '23'.");
    STAssertEquals(person.weight, (float)0.0, @"The parser should have created a Person instance with the attribute 'weight' equal to '0.0' since it does not exists in the json dataset and 0.0 is the default value for floats.");
    STAssertEquals(person.height, 0.0, @"The parser should have created a Person instance with the attribute 'height' equal to '0.0' since it does not exists in the json dataset and 0.0 is the default value for doubles.");
        STAssertNil(person.identityNumber, @"The parser should have created a Person instance with the attribute 'identityNumber' equal to 'nil' since it does not exists in the json dataset and nil is the default value for NSObjects.");
}

- (void)testBindClass_ReturnOnePersonInstanceWithThreeAttributes_Name_Age_Weight
{
    self.jsonDataSet = [self dataFromJsonDataSetWithName:_cmd];
    
    Person *person = [ROJsonBinder bindClass:[Person class] withJson:self.jsonDataSet];
    
    STAssertEqualObjects(person.name, @"Raphael", @"The parser should have created a Person instance with the attribute 'name' equal to 'Raphael'.");
    STAssertEquals(person.age, 25, @"The parser should have created a Person instance with the attribute 'age' equal to '25'.");
    STAssertEqualsWithAccuracy(person.weight, (float)70.53, FLOAT_MIN_ACCURACY, @"The parser should have created a Person instance with the attribute 'weight' equal to '70.53'.");
    STAssertEquals(person.height, 0.0, @"The parser should have created a Person instance with the attribute 'height' equal to '0.0' since it does not exists in the json dataset and 0.0 is the default value for doubles.");
    STAssertNil(person.identityNumber, @"The parser should have created a Person instance with the attribute 'identityNumber' equal to 'nil' since it does not exists in the json dataset and nil is the default value for NSObjects.");
}

- (void)testBindClass_ReturnOnePersonInstanceWithFiveAttributes_Name_Age_Weight_Height_IdentityNumber
{
    self.jsonDataSet = [self dataFromJsonDataSetWithName:_cmd];
    
    Person *person = [ROJsonBinder bindClass:[Person class] withJson:self.jsonDataSet];
    
    STAssertEqualObjects(person.name, @"Raphael Oliveira", @"The parser should have created a Person instance with the attribute 'name' equal to 'Raphael Oliveira'.");
    STAssertEquals(person.age, 31, @"The parser should have created a Person instance with the attribute 'age' equal to '31'.");
    STAssertEqualsWithAccuracy(person.weight, (float)71.897, FLOAT_MIN_ACCURACY, @"The parser should have created a Person instance with the attribute 'weight' equal to '71.897'.");
    STAssertEqualsWithAccuracy(person.height, 1.89, FLOAT_MIN_ACCURACY, @"The parser should have created a Person instance with the attribute 'height' equal to '1.89'.");
    STAssertEqualObjects(person.identityNumber, [NSNumber numberWithLongLong:85340506193], @"The parser should have created a Person instance with the attribute 'identityNumber' equal to '85340506193'.");

}


@end
